'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var React = require('react');
var index_cjs = require('../integration/index.cjs');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var React__default = /*#__PURE__*/_interopDefaultLegacy(React);

/**
 * The React `<Partytown/>` component should be placed within the `<head>`
 * of the document.
 *
 * @public
 */
const Partytown = (props = {}) => {
    React.useEffect(() => {
        if (typeof document !== 'undefined' && document.querySelector) {
            if (!document.querySelector(`script[data-pt-script="lib"]`)) {
                // Client side that should run on the document,
                // SSR/Static script does not exist, this must be client only
                // append the script script that should have run and append to the head
                const scriptElm = document.createElement('script');
                scriptElm.dataset.ptScript = 'lib';
                scriptElm.innerHTML = index_cjs.partytownSnippet(props);
                document.head.appendChild(scriptElm);
            }
        }
    }, []);
    // `dangerouslySetInnerHTML` only works for scripts rendered as HTML from SSR/Static.
    // Added code will set the [type="data-pt-script"] attribute with the "lib" id.
    // If this code renders as SSR/Static HTML, then it'll execute and add the attribute
    // which will tell the Client JS of the component to NOT add the same script to the head.
    const innerHTML = `document.currentScript.dataset.ptScript="lib";` + index_cjs.partytownSnippet(props);
    return React__default["default"].createElement("script", { suppressHydrationWarning: true, dangerouslySetInnerHTML: { __html: innerHTML } });
};

exports.Partytown = Partytown;
