'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var fs = require('fs');
var path = require('path');
var url = require('url');
var util = require('util');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var fs__default = /*#__PURE__*/_interopDefaultLegacy(fs);

const copyFile = util.promisify(fs__default["default"].copyFile);
const mkdir = util.promisify(fs__default["default"].mkdir);
const readdir = util.promisify(fs__default["default"].readdir);
const stat = util.promisify(fs__default["default"].stat);
const __dirname$1 = path.dirname(url.fileURLToPath((typeof document === 'undefined' ? new (require('u' + 'rl').URL)('file:' + __filename).href : (document.currentScript && document.currentScript.src || new URL('index.cjs', document.baseURI).href))));
/**
 * Absolute path to the Partytown lib directory within the
 * `@builder.io/partytown` package.
 *
 * @public
 */
function libDirPath() {
    return path.resolve(__dirname$1, '..', 'lib');
}
/**
 * Utility to copy the Partytown library files to a destination on the server.
 * Partytown requires its library files, such as `partytown.js` to be served
 * as static files from the same origin. By default the library assumes all the
 * files can be found at `/~partytown/`, but this can be configured.
 *
 * This utility function is to make it easier to locate the source library files
 * and copy them to your server's correct location, for example: `./public/~partytown/`.
 *
 * By default, both the minified and debug builds are copied to the destination.
 * However, by setting the `debugDir` option to `false`, the debug directory will
 * not be copied.
 *
 * @public
 */
async function copyLibFiles(destDir, opts = {}) {
    if (typeof destDir !== 'string' || destDir.length === 0) {
        throw new Error('Missing destDir');
    }
    if (!path.isAbsolute(destDir)) {
        destDir = path.resolve(process.cwd(), destDir);
    }
    await copyLibDir(libDirPath(), destDir, opts);
    return {
        src: libDirPath(),
        dest: destDir,
    };
}
async function copyLibDir(srcDir, destDir, opts) {
    try {
        await mkdir(destDir, { recursive: true });
    }
    catch (e) { }
    const itemNames = await readdir(srcDir);
    await Promise.all(itemNames.map(async (srcName) => {
        const srcPath = path.resolve(srcDir, srcName);
        const destPath = path.resolve(destDir, srcName);
        const s = await stat(srcPath);
        if (s.isFile()) {
            await copyFile(srcPath, destPath);
        }
        else if (s.isDirectory()) {
            if (srcName === 'debug' && opts.debugDir === false) {
                return;
            }
            await copyLibDir(srcPath, destPath, opts);
        }
    }));
}

exports.copyLibFiles = copyLibFiles;
exports.libDirPath = libDirPath;
